@@ -134,6 +134,7 @@ std::map<const std::string, Keywords::KEYWORDS>::value_type("rate_parameters_pk"
 std::map<const std::string, Keywords::KEYWORDS>::value_type("rate_parameters_svd",          Keywords::KEY_RATE_PARAMETERS_SVD),
 std::map<const std::string, Keywords::KEYWORDS>::value_type("rate_parameters_hermanska",    Keywords::KEY_RATE_PARAMETERS_HERMANSKA),
 std::map<const std::string, Keywords::KEYWORDS>::value_type("mean_gammas",                  Keywords::KEY_MEAN_GAMMAS),
+std::map<const std::string, Keywords::KEYWORDS>::value_type("gas_binary_parameters",        Keywords::KEY_GAS_BINARY_PARAMETERS),
 std::map<const std::string, Keywords::KEYWORDS>::value_type("solution_mix", 	            Keywords::KEY_SOLUTION_MIX),
 std::map<const std::string, Keywords::KEYWORDS>::value_type("mix_solution", 	            Keywords::KEY_SOLUTION_MIX),
 std::map<const std::string, Keywords::KEYWORDS>::value_type("exchange_mix", 	            Keywords::KEY_EXCHANGE_MIX),
@@ -228,7 +229,8 @@ std::map<Keywords::KEYWORDS, const std::string>::value_type(Keywords::KEY_REACTI
 std::map<Keywords::KEYWORDS, const std::string>::value_type(Keywords::KEY_RATE_PARAMETERS_PK,		    "RATE_PARAMETERS_PK"),
 std::map<Keywords::KEYWORDS, const std::string>::value_type(Keywords::KEY_RATE_PARAMETERS_SVD,		    "RATE_PARAMETERS_SVD"),
 std::map<Keywords::KEYWORDS, const std::string>::value_type(Keywords::KEY_RATE_PARAMETERS_HERMANSKA,    "RATE_PARAMETERS_HERMANSKA"),
-std::map<Keywords::KEYWORDS, const std::string>::value_type(Keywords::KEY_MEAN_GAMMAS,		            "RATE_MEAN_GAMMAS"),
+std::map<Keywords::KEYWORDS, const std::string>::value_type(Keywords::KEY_MEAN_GAMMAS,		            "MEAN_GAMMAS"),
+std::map<Keywords::KEYWORDS, const std::string>::value_type(Keywords::KEY_GAS_BINARY_PARAMETERS,		"GAS_BINARY_PARAMETERS"),
 std::map<Keywords::KEYWORDS, const std::string>::value_type(Keywords::KEY_SOLUTION_MIX,		            "SOLUTION_MIX"),
 std::map<Keywords::KEYWORDS, const std::string>::value_type(Keywords::KEY_EXCHANGE_MIX,		            "EXCHANGE_MIX"),
 std::map<Keywords::KEYWORDS, const std::string>::value_type(Keywords::KEY_GAS_PHASE_MIX,		        "GAS_PHASE_MIX"),
