@@ -1,6 +1,14 @@
 #include <stdio.h>
 #include "Use.h"
 
+#if defined(PHREEQCI_GUI)
+#ifdef _DEBUG
+#define new DEBUG_NEW
+#undef THIS_FILE
+static char THIS_FILE[] = __FILE__;
+#endif
+#endif
+
 cxxUse::cxxUse()
 {
 	this->init();
